#version 430 core

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	vec4 ShadowPosition;
} vert;


uniform sampler2D diffuseTexture;
uniform sampler2D normalTexture;
uniform sampler2D shadowMap;

uniform vec3 lightPosition;

layout(location = 0) out vec4 diffuseColor;
layout(location = 1) out vec4 normalColor;
layout(location = 2) out vec4 shadowIntensity;


float ShadowCalculation(vec4 fragPosLightSpace, vec3 n)
{
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    projCoords = projCoords * 0.5 + 0.5;
    float dotProd = dot(n, lightPosition);
    if(dotProd < 0){
       return 0.5f;
    }
    float currentDepth = projCoords.z;
    float bias = max(0.001 * (1.0 - abs(dotProd)), 0.0024);

    float shadow = 0.0;
    vec2 texelSize = 1.0 / textureSize(shadowMap, 0);

    for(int x = -1; x <= 1; x++)
    {
        for(int y = -1; y <= 1; y++)
        {
            float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
            shadow += currentDepth - bias > pcfDepth ? 0.5 : 0.0;
        }
    }

    shadow /= 9.0;
    
    return 1.0 - shadow;
}

vec3 getNormalFromMap(vec3 Normal)
{
    vec3 tangentNormal = texture(normalTexture, vert.uv).rgb * 2.0 - 1.0;

    vec3 Q1  = dFdx(vert.position_world);
    vec3 Q2  = dFdy(vert.position_world);
    vec2 st1 = dFdx(vert.uv);
    vec2 st2 = dFdy(vert.uv);

    vec3 N   = normalize(Normal);
    vec3 T  = normalize(Q1*st2.t - Q2*st1.t);
    vec3 B  = -normalize(cross(N, T));
    mat3 TBN = mat3(T, B, N);

    return normalize(TBN * tangentNormal);
}

void main() {	
	vec3 n = normalize(vert.normal_world);
    n = getNormalFromMap(n);

    float shadow = ShadowCalculation(vert.ShadowPosition, n);

    vec3 projCoords = vert.ShadowPosition.xyz / vert.ShadowPosition.w;
    projCoords = projCoords * 0.5 + 0.5;
    float shadowValue = texture(shadowMap, projCoords.xy).r;
    shadowValue = shadowValue * 100.0;
    normalColor = vec4( n , 1.0 ) ;
    vec3 rawColor = texture(diffuseTexture, vert.uv).rgb;
    diffuseColor = vec4( rawColor * shadow , 1.0 );
    shadowIntensity = vec4(vec3(shadow), 1.0);
	// float ndc = gl_FragCoord.z * 2.0 - 1.0; 
	// float linearDepth = (2.0 * near * far) / (far + near - ndc * (far - near));
    
	// color = vec4( vec3( linearDepth / far ), 1.0 );
	// color = vec4( texture(diffuseTexture, vert.uv).rgb , 1.0 );
    
}