#version 430 core

out vec4 FragColor;

in vec2 TexCoords;
uniform sampler2D diffuseTexture;
uniform sampler2D normalTexture;

const float offset = 1.0 / 400.0;  

void main()
{
    vec2 offsets[9] = vec2[](
        vec2(-offset,  offset), // top-left
        vec2( 0.0f,    offset), // top-center
        vec2( offset,  offset), // top-right
        vec2(-offset,  0.0f),   // center-left
        vec2( 0.0f,    0.0f),   // center-center
        vec2( offset,  0.0f),   // center-right
        vec2(-offset, -offset), // bottom-left
        vec2( 0.0f,   -offset), // bottom-center
        vec2( offset, -offset)  // bottom-right    
    );

    float sobel_horizontal_kernel[9] = float[](
        1, 1, 1,
        1, -8, 1,
        1, 1, 1
    );

        float blur_kernel[9] = float[](
        1.0 / 16, 2.0 / 16, 1.0 / 16,
        2.0 / 16, 4.0 / 16, 2.0 / 16,
        1.0 / 16, 2.0 / 16, 1.0 / 16
    );
    
    vec3 sampleTex_normal[9];
    vec3 sampleTex_diffuse[9];


    for(int i = 0; i < 9; i++)
    {
        sampleTex_normal[i] = vec3(texture(normalTexture, TexCoords.st + offsets[i]));
        sampleTex_diffuse[i] = vec3(texture(diffuseTexture, TexCoords.st + offsets[i]));
    }
    vec3 col = vec3(0.0);
    vec3 diffCol = vec3(0.0);
    for(int i = 0; i < 9; i++){
        col += sampleTex_normal[i] * sobel_horizontal_kernel[i];
        diffCol += sampleTex_diffuse[i] * blur_kernel[i];
    }
    if( ( col.r + col.g + col.b ) > 0.5){
        col = vec3( 0.0);
    } else {
        col = vec3( 1.0);
    }
    diffCol = texture(diffuseTexture, TexCoords.st).rgb;
    FragColor = vec4( diffCol * col, 1.0 );

}  